class Constants:
    APP_NAME = "Domestica"
    CONFIG_FILE = "config.json"
    COOKIES_FILE = "cookies.json"


class Messages:
    VALUE_DOES_NOT_MATCH = "[bold red]Error![/bold red] [magenta]The two entered values do not match.[/magenta]"
